from gwdc_python.files.filters import filter_file_list
from . import identifiers


def default_filter(file_list):
    """Takes an input file list and returns a subset of that file list containing:

    - Any HTML file
    - Any file ending with '_config_complete.ini'
    - Any PNG files in the 'data' directory
    - Any PNG files in the 'result' directory
    - Any file in the 'result' directory ending in '_merge_result.json', or '_result.json' if there is no merged file

    Parameters
    ----------
    file_list : .FileReferenceList
        A list of FileReference objects which will be filtered

    Returns
    -------
    .FileReferenceList
        Subset of the input FileReferenceList containing only the paths that match the above default file criteria
    """
    # Get png files in data dir
    data_png_file_list, file_list = filter_file_list(identifiers.data_png_file, file_list)

    # Get png files in result dir
    result_png_file_list, file_list = filter_file_list(identifiers.result_png_file, file_list)

    # Get complete config file
    config_file_list, file_list = filter_file_list(identifiers.config_file, file_list)

    # Get index html file
    html_file_list, file_list = filter_file_list(identifiers.html_file, file_list)

    # Get merged json file in result dir
    result_json_file_list, file_list = filter_file_list(identifiers.result_merged_json_file, file_list)

    # If merged json doesn't exist, get result json file in result dir
    if not result_json_file_list:
        result_json_file_list, file_list = filter_file_list(identifiers.result_json_file, file_list)

    return data_png_file_list + result_png_file_list + config_file_list + html_file_list + result_json_file_list


def config_filter(file_list):
    """Takes an input file list and returns a subset of that file list containing:

    - Any file ending with '_config_complete.ini'

    Parameters
    ----------
    file_list : .FileReferenceList
        A list of FileReference objects which will be filtered

    Returns
    -------
    .FileReferenceList
        Subset of the input FileReferenceList containing only the paths that match the above config file criteria
    """
    return filter_file_list(identifiers.config_file, file_list)[0]


def png_filter(file_list):
    """Takes an input file list and returns a subset of that file list containing:

    - Any PNG file

    Parameters
    ----------
    file_list : .FileReferenceList
        A list of FileReference objects which will be filtered

    Returns
    -------
    .FileReferenceList
        Subset of the input FileReferenceList containing only the paths that match the above png file criteria
    """
    return filter_file_list(identifiers.png_file, file_list)[0]


def corner_plot_filter(file_list):
    """Takes an input file list and returns a subset of that file list containing:

    - Any file ending in '_corner.png'

    Parameters
    ----------
    file_list : .FileReferenceList
        A list of FileReference objects which will be filtered

    Returns
    -------
    .FileReferenceList
        Subset of the input FileReferenceList containing only the paths that match the above corner plot file criteria
    """
    return filter_file_list(identifiers.corner_plot_file, file_list)[0]


def result_json_filter(file_list):
    """Takes an input file list and returns a subset of that file list containing:

    - Any file in the 'result' directory ending in '_merge_result.json'
    - Or, any file in the 'result' directory ending in '_result.json'

    Parameters
    ----------
    file_list : .FileReferenceList
        A list of FileReference objects which will be filtered

    Returns
    -------
    .FileReferenceList
        Subset of the input FileReferenceList containing only the paths that match the merged json file criteria
    """
    # Get merged json file in result dir
    result_json_file_list, _ = filter_file_list(identifiers.result_merged_json_file, file_list)

    # If merged json doesn't exist, get result json file in result dir
    if not result_json_file_list:
        result_json_file_list, _ = filter_file_list(identifiers.result_json_file, file_list)

    return result_json_file_list


def sort_file_list(file_list):
    """Sorts a file list based on the 'path' key of the dicts. Primarily used for equality checks.

    Parameters
    ----------
    file_list : .FileReferenceList
        A list of FileReference objects which will be filtered

    Returns
    -------
    .FileReferenceList
        A FileReferenceList containing the same members as the input,
        sorted by the path attribute of the FileReference objects
    """
    return sorted(file_list, key=lambda f: f.path)



import functools
from gwdc_python.exceptions import GWDCAuthenticationError


class GWCloudAuthenticationError(Exception):
    def __init__(self):
        super().__init__(
            """
Your API token does not exist, make sure it is correct!

Please read the API token documentation:
https://gwcloud-python.readthedocs.io/en/latest/gettingstarted.html#getting-access

Alternatively, head straight to https://gwcloud.org.au/auth/api-token to create one.
            """
        )


def custom_error_handler(f):
    @functools.wraps(f)
    def wrapper(*args, **kwargs):
        try:
            return f(*args, **kwargs)
        except GWDCAuthenticationError:
            raise GWCloudAuthenticationError
    return wrapper



from .utils import file_filters
from .event_id import EventID

from gwdc_python.jobs import JobBase
from gwdc_python.logger import create_logger

logger = create_logger(__name__)


class BilbyJob(JobBase):
    """
    BilbyJob class is useful for interacting with the Bilby jobs returned from a call to the GWCloud API.
    It is primarily used to store job information and obtain files related to the job.

    Parameters
    ----------
    client : ~gwcloud_python.gwcloud.GWCloud
        A reference to the GWCloud object instance from which the BilbyJob was created
    job_id : str
        The id of the Bilby job, required to obtain the files associated with it
    name : str
        Job name
    description : str
        Job description
    user : str
        User that ran the job
    event_id : dict
        Event ID associated with job, should have keys corresponding to an
        :class:`~.EventID` object
    job_status : dict
        Status of job, should have 'name' and 'date' keys corresponding to the status code and when it was produced
    kwargs : dict, optional
        Extra arguments, stored in `other` attribute
    """

    FILE_LIST_FILTERS = {
        'default': file_filters.default_filter,
        'config': file_filters.config_filter,
        'png': file_filters.png_filter,
        'corner_plot': file_filters.corner_plot_filter,
        'result_json': file_filters.result_json_filter
    }

    def __init__(self, client, job_id, name, description, user, event_id, job_status, **kwargs):
        super().__init__(client, job_id, name, description, user, job_status)
        self.event_id = EventID(**event_id) if event_id else None
        self.other = kwargs

    def _update_job(self, **kwargs):
        query = """
            mutation BilbyJobEventIDMutation($input: UpdateBilbyJobMutationInput!) {
                updateBilbyJob(input: $input) {
                    result
                }
            }
        """

        variables = {
            "input": {
                "job_id": self.job_id,
                **kwargs
            }
        }

        return self.client.request(query=query, variables=variables)

    def set_name(self, name):
        """Set the name of a Bilby Job

        Parameters
        ----------
        event_id : str
            The new name
        """

        data = self._update_job(name=str(name))
        self.name = name
        logger.info(data['update_bilby_job']['result'])

    def set_description(self, description):
        """Set the description of a Bilby Job

        Parameters
        ----------
        event_id : str
            The new description
        """

        data = self._update_job(description=str(description))
        self.description = description
        logger.info(data['update_bilby_job']['result'])

    def set_event_id(self, event_id=None):
        """Set the Event ID of a Bilby Job

        Parameters
        ----------
        event_id : EventID or str, optional
            The desired Event ID, by default None
        """

        if isinstance(event_id, EventID):
            new_event_id = event_id.event_id
        elif isinstance(event_id, str):
            new_event_id = event_id
        elif event_id is None:
            new_event_id = ''
        else:
            raise Exception('Parameter event_id must be an EventID, a string or None')

        data = self._update_job(event_id=new_event_id)
        self.event_id = self.client.get_event_id(event_id=new_event_id)
        logger.info(data['update_bilby_job']['result'])



from .gwcloud import GWCloud
from .bilby_job import BilbyJob
from .event_id import EventID

from gwdc_python.files import FileReference, FileReferenceList
from gwdc_python.helpers import TimeRange, Cluster, JobStatus


try:
    from importlib.metadata import version
except ModuleNotFoundError:
    from importlib_metadata import version
__version__ = version('gwcloud_python')
